#!/usr/local/bin/perl

#*------------------------------------------------*
#* COPYRIGHT (C) 1996                             *
#*         Northeast Parallel Architectures Center*
#*         Syracuse University                    *
#*------------------------------------------------*

#$Id$
#$Log$
#Author: Kivanc Dincer, NPAC 1996.

require 'query-string.pl';
require 'html-util.pl';
require 'set-env.pl';

#>-----------------------------------------------------------------<
#> Function    : prints out an ascii file's contents onto stdout.
#> In Arguments: file's name ($FName)
#> Side Effect : on stdout.
#>-----------------------------------------------------------------<
sub Print_File{
   local($FName) = @_;
   open(InFile, $FName) || do htmlDie("Print_File:$!");
#   print "\n$FName:\n";
   while (<InFile>) {
      print $_;
   }
   close(InFile);
}

#>-----------------------------------------------------------------<
#> Function    : prints HTML page header for the monitor page.
#> In Arguments: none.
#> Side Effect : on stdout.
#>-----------------------------------------------------------------<
sub Print_Page_Header {
  print "Content-type: text/html\n\n";
  print "<HTML>\n";
  print "<HEAD>\n";
  print "<TITLE></TITLE>\n";
  print "</HEAD>\n";
  print '<BODY BGCOLOR="white" TEXT="black">';
  print "<FORM>";
  print '<CENTER>
         <H2><FONT COLOR="blue">Results of Submitted Process</H2> 
         </CENTER><HR></FONT>';
}

#>-----------------------------------------------------------------<
#> Function    : manages the HPF to F77+MP  translation by using Makefile.
#> In Arguments: names of the HPF and F77 files.
#> Side Effect : generates .f file.
#>-----------------------------------------------------------------<
sub DoCompile_byMake {    
   local($DemoDir, $targetF) = @_;
   local($cmnd); 
#>
#> Print information about the ongoing process.
#>
   $cmnd = "$targetF"; 
   print STDOUT "<i>Compiling: (make $cmnd) ...</i><BR>";
   print STDOUT "<TEXTAREA NAME=\"DISPLAY\" ROWS=\"10\" COLS=\"70\">";
   $stat = system("cd $DemoDir; $cmnd >compile.log  2>&1");
   $stat >>= 8;
   $stat = system("cd $DemoDir; make $cmnd");
   $stat >>= 8;
   print "\$stat is $stat and \$? is $? \n";
   &EndSwitchOut();
   
   do Print_File("stdout.dat");      
   do Print_File("stderr.dat");
   print "</TEXTAREA>";
   print "<HR>";
}

#>-----------------------------------------------------------------<
#> Function    : manages the execution process.
#> In Arguments: name of the target file and number of nodes required.
#> Side Effect : output is shown on an edit window.
#>-----------------------------------------------------------------<
sub DoExecute {
   local($numNodes, $DemoDir, $target, $inputF, $outputfile ) = @_;
   local($cmnd, $execfile);

   print "<br><i>";

   $execfile = "$username-$target";
   if ($numNodes eq 1) {$maxnodes = 1;} else {$maxnodes=8;}
   for ($i=1; $i<=$maxnodes; $i++) { 
      $cpcmnd = "/usr/ucb/rcp $DemoDir/$target kestrel$i:/scratch/$execfile"; 
      $stat = system("cd $DemoDir; $cpcmnd >copy.log  2>&1");
   }
   $cmnd = "mpirun -np $numNodes /scratch/$execfile"; 
   if ($compiler eq "F77") {
       print "Executing (Fortran 77+MPI) code ($cmnd) on $numNodes node";
   }
   else {
       print "Executing (C+MPI) code ($cmnd) on $numNodes node";
   }
   print "s " unless ($numNodes == 1); 
   print "of the DEC Alphafarm ...";
   print " </i><BR>";
   print STDOUT "<TEXTAREA NAME=\"DISPLAY3\" ROWS=\"10\" COLS=\"60\"
          WRAP=\"VIRTUAL\">";
   
   $stat=system("limit cputime 3:00");
   $stat = system("cd $DemoDir; $cmnd >execute.log  2>&1");
   do Print_File("$DemoDir/execute.log");
   if ($stat) {
      print "Execution (i.e.,$cmnd) failed :-> \n $! \n";
   }
   else {
      print "Execution completed successfully! \n";
   }

   if ($outputfile) {
      #print "cp execute.log $dirname/$outputfile";
       $stat=system("cd $DemoDir; cp execute.log $dirname/$outputfile > cp.log 2>&1");
       if ($stat) {
	   print "\nOutput File could not be written!->\n $! \n";
       } 
   } 
   print "</TEXTAREA><HR> \n";
}

#>-----------------------------------------------------------------<
#> Function    : manages the C+MPI or F77+MPI compilation by using mpi scripts.
#> In Arguments: names of the C and F77 files in $files list,
#>               name of TARGET and name of user's lab directory.
#> Side Effect : generates .f file.
#>-----------------------------------------------------------------<
sub DoCompile_byCompiler {    
   local($ProgDir, $target, $files) = @_;

   local($flags) = "-lm";
   
   if ($compiler eq "F77") {
       $cmnd .= "mpif77 -o $target $files $flags";
   } elsif ($compiler eq "C") {
       $cmnd .= "mpicc  -o $target $files $flags"; 
   } else {
       &htmlDie("Compile: Wrong arguments cmnd=$cmnd.");
   }
   print "<i>Compiling: ($cmnd) ...</i><BR>";
   $cmnd = "/home/B7A/pcrc_hpf/mpi-scripts/" . $cmnd; 
   print STDOUT "<TEXTAREA NAME=\"DISPLAY3\" ROWS=\"10\" COLS=\"60\" 
         WRAP=\"VIRTUAL\">";

   $stat = system("cd $DemoDir; $cmnd >compile.log  2>&1");
   $stat >>= 8;

   do Print_File("$DemoDir/compile.log");
   if ($stat) {
      print "DEBUG: \$stat is $stat and \$? is $? \n";
      print "Exiting ... \n";
      print "</TEXTAREA>";
#      die; 
   }
   else {
      print "Compilation is successful! \n";
      print "</TEXTAREA>";
   } 
   print "<HR>";

}

#>-----------------------------------------------------------------<
#> Function    : sets all flags and variables used in this script 
#>               according to user's selections.
#> In Arguments: $MY_DATA
#> Side Effect : All variables/flags given below are set.
#>-----------------------------------------------------------------<
sub Set_Compiler_Flags{
    local($i, $fldname);

    $numNodes      = $MYDATA{'MP_PROCS'};

    $compile       = ($MYDATA{'TASK.COMPILE'}) ? 1 : 0;
    $compiler      = $MYDATA{'COMPILER_TYPE'};
    $dirname      = $MYDATA{'FILENAME'};
    $dirname = substr($dirname, 1, -1);
    $dirname = substr($dirname, 0, rindex($dirname,"/")); 
    $filename= $MYDATA{'FILENAME'};
    #$filename= substr($filename, rindex($filename,"/")+1,length($filename));
    $execute       = ($MYDATA{'TASK.RUN'})     ? 1 : 0;
    $executable_name= $MYDATA{'EXECUTABLE_NAME'};
    $inputfile     = $MYDATA{'INPUT_FILE'}; 
    $outputfile    = $MYDATA{'OUTPUT_FILE'};
}
#>-----------------------------------------------------------------<
#> Function    : prints debugging statements.
#> In Arguments: chosen variables/flags of the script.
#> Side Effect : Chosen variables/flags are printed out.
#>-----------------------------------------------------------------<
sub Print_Debugging_Stmts {
    print "A few debugging stmts:";
    print "<blockquote>";

    print "directory: $DemoDir <br>";
    print "username: $username <br>";

    print "compile:  $compile <br>";
    print "execute:  $execute <br>";
    print "numNodes: $numNodes <br>";

    print "compiler: $compiler <BR>";
    print "filename: $filename <BR>";
    print "executable: $executable_name<BR>";

    print "MP_ variables: $numNodes <BR> ";
    print "inputfile:     $inputfile <BR>";
    print "outputfile:    $outputfile <BR>";

    print "</blockquote>";
    print "End of the debugging stmts<p>";
}

#
# -------------------- M A I N --------------------
#
   &Read_Query_String();
   &Parse_Query_String();
   &Define_Constants();

    &Print_Page_Header();
#>
#>- Set Compiler Flags   
#>
    &Set_Compiler_Flags();
#>
#> A few debugging stmts
#>
    #&Print_Debugging_Stmts();
    $targetFile = $executable_name;
    $ProgDir = $DemoDir . "/" . $dirname . "/";
   if ($compile) {
       #&DoCompile_byMake($DemoDir, $targetFile);
       &DoCompile_byCompiler($ProgDir, $targetFile, $filename);
   }

   if ($execute) {
           &DoExecute($numNodes, $DemoDir, $targetFile, 
                  $inputfile, $outputfile);
   }
print<<END;
<CENTER>
<SCRIPT>
<!-----
    document.write('<input type="button" value=" Close Window " onClick="parent.close(this)">');
//----->
</SCRIPT>
</CENTER>
END
  
   print "</BODY>\n";
   print "</HTML>\n";

