/*
 * File         : Intracommm.java
 * Author       : Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/08/26 18:49:58 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

package mpi;
import mpi.*;

public class Intracomm extends Comm {

  public Intracomm() {}
  public Intracomm(int Type) { GetComm(Type);}
  public Intracomm(long _handle) { handle=_handle; }

  public Object clone()
  {  return new Intracomm(dup());
     }

  public Intracomm Split(int colour, int key)
  {  return new Intracomm(split(colour,key));
     }
  private native long split(int colour, int key);

  private native long creat(Group group);
  public Intracomm Creat(Group group)
  { long handle=creat(group);
    if(handle == 0)
        return null;
    return new Intracomm(handle);
    }

  // Collective Communication

  public native void Barrier();

  public native void bcast(Object buf, int offset, int count, Datatype
    type,int root);
  public void Bcast(Object buf, int offset, int count, Datatype
	type,int root)
  { bcast(buf,offset*type.baseSize(),count,type,root); 
     }

  private native void gather(Object sendbuf, int sendoffset, int sendcount,
	Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
	Datatype recvtype, int root);
  public void Gather(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype, int root)
  { gather(sendbuf,sendoffset*sendtype.baseSize(), sendcount,sendtype,
	recvbuf, recvoffset*recvtype.baseSize(), recvcount,recvtype,root);
	}

  private native void gatherv(Object sendbuf, int sendoffset, int sendcount,
	Datatype sendtype, Object recvbuf, int recvoffset, int [] recvcount,
	int [] displs, Datatype recvtype, int root);
  public void Gatherv(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int [] recvcount,
    int [] displs, Datatype recvtype, int root)
  { gatherv(sendbuf,sendoffset*sendtype.baseSize(), sendcount, sendtype,
	recvbuf, recvoffset*recvtype.baseSize(),recvcount,displs,recvtype, root);
	}

  private native void scatter(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype, int root);
  public void Scatter(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype, int root)
  { scatter(sendbuf, sendoffset*sendtype.baseSize(), sendcount, sendtype, recvbuf, recvoffset*recvtype.baseSize(), recvcount, recvtype, root);
	}

  private native void scatterv(Object sendbuf, int sendoffset, 
	int [] sendcount,int [] displs,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype, int root);
  public void Scatterv(Object sendbuf, int sendoffset,
    int [] sendcount,int [] displs,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype, int root)
  {scatterv( sendbuf, sendoffset*sendtype.baseSize(), 
	sendcount, displs,
     sendtype, recvbuf, recvoffset*recvtype.baseSize(),  recvcount,
    recvtype,  root);
	}

  private native void allgather(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype);
  public void Allgather(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype)
  { allgather(sendbuf, sendoffset*sendtype.baseSize(), sendcount,
    sendtype, recvbuf, recvoffset*recvtype.baseSize(), recvcount,
    recvtype);
	}

  private native void allgatherv(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int [] recvcount,
    int [] displs, Datatype recvtype);
  public void Allgatherv(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int [] recvcount,
    int [] displs, Datatype recvtype)
  {allgatherv(sendbuf, sendoffset*sendtype.baseSize(), sendcount,
    sendtype, recvbuf, recvoffset*recvtype.baseSize(), recvcount,
    displs, recvtype);
	}

  private native void alltoall(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype);
  public void Alltoall(Object sendbuf, int sendoffset, int sendcount,
    Datatype sendtype, Object recvbuf, int recvoffset, int recvcount,
    Datatype recvtype)
  { alltoall(sendbuf, sendoffset*sendtype.baseSize(), sendcount,
    sendtype, recvbuf, recvoffset*recvtype.baseSize(), recvcount,
    recvtype);
	}

  private native void alltoallv(Object sendbuf, int sendoffset, int [] sendcount,
	int [] sdispls,
    Datatype sendtype, Object recvbuf, int recvoffset, int [] recvcount,
    int [] displs, Datatype recvtype);
  public void Alltoallv(Object sendbuf, int sendoffset, int [] sendcount,
    int [] sdispls,
    Datatype sendtype, Object recvbuf, int recvoffset, int [] recvcount,
    int [] displs, Datatype recvtype)
  {alltoallv(sendbuf, sendoffset*sendtype.baseSize(), sendcount,
	sdispls,
    sendtype, recvbuf, recvoffset*recvtype.baseSize(), recvcount,
    displs,recvtype);
	}

  private native void reduce(Object sendbuf, int sendoffset, 
    Object recvbuf, int recvoffset, int count, Datatype datatype, Op op,
    int root);
  public void Reduce(Object sendbuf, int sendoffset,
    Object recvbuf, int recvoffset, int count, Datatype datatype, Op op,
    int root)
  { reduce(sendbuf, sendoffset*datatype.baseSize(), 
    recvbuf, recvoffset*datatype.baseSize(), count, datatype, op,
    root);
	}

  private native void allreduce(Object sendbuf, int sendoffset, 
    Object recvbuf, int recvoffset, int count, Datatype datatype, Op op);
  public void Allreduce(Object sendbuf, int sendoffset,
    Object recvbuf, int recvoffset, int count, Datatype datatype, Op op)
  {allreduce(sendbuf, sendoffset*datatype.baseSize(), 
    recvbuf, recvoffset*datatype.baseSize(), count, datatype, op);
	}

  private native void reduce_scatter(Object sendbuf, int sendoffset, 
    Object recvbuf, int recvoffset, int [] recvcounts, 
	Datatype datatype, Op op);
  public void Reduce_scatter(Object sendbuf, int sendoffset,
    Object recvbuf, int recvoffset, int [] recvcounts,
    Datatype datatype, Op op)
  {reduce_scatter(sendbuf, sendoffset*datatype.baseSize(),
    recvbuf, recvoffset*datatype.baseSize(), recvcounts, 
	datatype, op);
	}

  private native void scan(Object sendbuf, int sendoffset, 
    Object recvbuf, int recvoffset, int count, Datatype datatype, Op op);
  public void Scan(Object sendbuf, int sendoffset,
    Object recvbuf, int recvoffset, int count, Datatype datatype, Op op)
  { scan(sendbuf, sendoffset*datatype.baseSize(), 
     recvbuf, recvoffset*datatype.baseSize(), count, datatype, op);
	}

  // Topology Constructors
  public Cartcomm Create_cart(int [] dims, boolean [] periods, 
	boolean reorder)
  { return new Cartcomm(GetCart(dims,periods,reorder)); }
  public native long GetCart(int[] dims, boolean[] periods, boolean reorder);

  public Graphcomm Create_graph(int [] index, int [] edges,
    boolean reorder)
  { return new Graphcomm(GetGraph(index,edges,reorder)); }
  public native long GetGraph(int[] index,int []edges, boolean reorder);

}
