/*
 * File         : MPI.java
 * Author       : Sung-Hoon Ko, Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/08/26 18:49:59 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

package mpi;
import mpi.*;

public class MPI {

  static int MAX_PROCESSOR_NAME = 256;

//  static public Comm COMM_WORLD;
  static public Intracomm COMM_WORLD;
  static public Comm COMM_SELF;

  static public int GRAPH, CART;
  static public int ANY_SOURCE, ANY_TAG;

  static public Op MAX, MIN, SUM, PROD, LAND, BAND,
    LOR, BOR, LXOR, BXOR, MINLOC, MAXLOC;
  static public Datatype BYTE, CHAR, SHORT, BOOLEAN, 
    INT, LONG, FLOAT, DOUBLE, PACKED, LB, UB;
  static public Datatype SHORT2, INT2, LONG2, FLOAT2, DOUBLE2;

  static public Request REQUEST_NULL;
  static public Group GROUP_EMPTY;

  static public int PROC_NULL;
  static public int BSEND_OVERHEAD;
  static public int UNDEFINED;
  static public int IDENT, CONGRUENT, SIMILAR, UNEQUAL;
  static public int TAG_UB, HOST, IO;

  static Errhandler ERRORS_ARE_FATAL, ERRORS_RETURN;

  static private native void InitNative(String[] args);
  static private native void SetConstant();
  static public void Init(String[] args) {
    InitNative(args);
    MAX = new Op(Op.MAX);
    MIN = new Op(Op.MIN);
    SUM = new Op(Op.SUM);
    PROD = new Op(Op.PROD);
    LAND = new Op(Op.LAND);
    BAND = new Op(Op.BAND);
    LOR = new Op(Op.LOR);
    BOR = new Op(Op.BOR);
    LXOR = new Op(Op.LXOR);
    BXOR = new Op(Op.BXOR);
    MINLOC = new Op(Op.MINLOC);
    MAXLOC = new Op(Op.MAXLOC);
    BYTE = new Datatype(Datatype.BYTE);
    CHAR = new Datatype(Datatype.CHAR);
    SHORT = new Datatype(Datatype.SHORT);
    BOOLEAN = new Datatype(Datatype.BOOLEAN);
    INT = new Datatype(Datatype.INT);
    LONG = new Datatype(Datatype.LONG);
    FLOAT = new Datatype(Datatype.FLOAT);
    DOUBLE = new Datatype(Datatype.DOUBLE);
    PACKED = new Datatype(Datatype.PACKED);
    LB = new Datatype(Datatype.LB);
    UB = new Datatype(Datatype.UB);

    GROUP_EMPTY = new Group(Group.EMPTY);	
    REQUEST_NULL = new Request(Request.NULL);
    
    // Constant
    SetConstant(); 

    ERRORS_ARE_FATAL= new Errhandler(Errhandler.FATAL);
    ERRORS_RETURN = new Errhandler(Errhandler.RETURN);       

//    COMM_WORLD = new Comm(Comm.WORLD);
    COMM_WORLD = new Intracomm(Intracomm.WORLD);
  }

  static public native void Finalize();

  static public native double Wtime();
  static public native double Wtick();

  static private native int Get_processor_name(byte[] buf);
  static public String Get_processor_name() {
    byte[] buf = new byte[MAX_PROCESSOR_NAME];
	int lengh=Get_processor_name(buf);
    //return new String(buf,0,0,lengh); //deprecated API in 1.1.x
    return new String(buf,0,lengh);
  }

  static public native boolean Initialized();

  // Buffer allocation
  static public native void Buffer_attach(byte[] buf);
  static public native void Buffer_detach(byte[] buf);
   
  static {
    System.loadLibrary("mpijava");
  }
}

